@@ -1,10 +1,10 @@
 /* A Scheme reader compiler for Guile.
 
-   Copyright (C) 2005, 2007, 2008  Ludovic Courts <ludo@gnu.org>
+   Copyright (C) 2005, 2007, 2008, 2009, 2012  Ludovic Courts <ludo@gnu.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -46,6 +46,13 @@ const scm_token_reader_spec_t scm_sharp_reader_standard_specs[] =
     SCM_TR_NUMBER_AND_RADIX,
     SCM_TR_GUILE_EXTENDED_SYMBOL,
     SCM_TR_SCSH_BLOCK_COMMENT,
+
+    /* The following syntax is supported by default in Guile 1.9 and later
+       only, but we always enable it, so that Guile-Reader can read the same
+       source regardless of the underlying version of Guile.  */
+    SCM_TR_R6RS_SYNTAX_QUOTE_QUASIQUOTE_UNQUOTE,
+    SCM_TR_SRFI62_SEXP_COMMENT,
+
     SCM_END_TOKENS
   };
 
@@ -63,12 +70,17 @@ scm_token_reader_spec_t scm_reader_standard_specs[] =
 			 "It is actually defined at reader's compile-"
 			 "time."),
 
-    SCM_TR_SEXP,
     SCM_TR_STRING,
-    SCM_TR_GUILE_NUMBER,
-    SCM_TR_GUILE_SYMBOL_LOWER_CASE,
-    SCM_TR_GUILE_SYMBOL_UPPER_CASE,
-    SCM_TR_GUILE_SYMBOL_MISC_CHARS,
+    SCM_TR_SEXP,
+
+    /* Guile 2.0 recognizes square brackets by default, so follow
+       its lead.  */
+    SCM_TR_SQUARE_BRACKET_SEXP,
+    SCM_TR_R6RS_NUMBER,
+    SCM_TR_R6RS_SYMBOL_LOWER_CASE,
+    SCM_TR_R6RS_SYMBOL_UPPER_CASE,
+    SCM_TR_R6RS_SYMBOL_MISC_CHARS,
+
     SCM_TR_QUOTE_QUASIQUOTE_UNQUOTE,
     SCM_TR_SEMICOLON_COMMENT,
 
@@ -133,11 +145,11 @@ SCM_DEFINE (scm_default_sharp_reader_token_readers,
 
 SCM_DEFINE (scm_make_guile_reader, "make-guile-reader", 0, 1, 1,
 	    (SCM fault_handler, SCM flags),
-	    "Make and return a new reader compatible with Guile's built-in "
-	    "reader.  This function call @code{make-reader} with "
-	    "@var{flags}.  Note that the sharp reader used by the returned "
-	    "reader is also instantiated using @var{flags}.  The value of "
-	    "@var{fault-handler} defaults to "
+	    "Make and return a new reader compatible with Guile 2.0's "
+	    "@code{read}, with its default settings.  This function calls "
+	    "@code{make-reader} with @var{flags}.  Note that the sharp "
+	    "reader used by the returned reader is also instantiated using "
+	    "@var{flags}.  The value of @var{fault-handler} defaults to "
 	    "@code{%reader-standard-fault-handler}.")
 #define FUNC_NAME s_scm_make_guile_reader
 {
@@ -148,8 +160,8 @@ SCM_DEFINE (scm_make_guile_reader, "make-guile-reader", 0, 1, 1,
   size_t code_size;
   void *buffer;
 
-  if (fault_handler == SCM_UNDEFINED)
-    fault_handler = scm_reader_standard_fault_handler_proc;
+  if (SCM_UNBNDP (fault_handler))
+    fault_handler = scm_variable_ref (scm_reader_standard_fault_handler_var);
 
   if (fault_handler != SCM_BOOL_F)
     SCM_VALIDATE_PROC (1, fault_handler);
@@ -158,7 +170,7 @@ SCM_DEFINE (scm_make_guile_reader, "make-guile-reader", 0, 1, 1,
 
   /* Build a brand new sharp reader.  Should be enough for both readers.  */
   buffer = scm_malloc (SHARP_READER_SIZE + TOP_LEVEL_READER_SIZE);
-  c_sharp_reader = scm_c_make_reader (buffer + TOP_LEVEL_READER_SIZE,
+  c_sharp_reader = scm_c_make_reader ((char *) buffer + TOP_LEVEL_READER_SIZE,
 				      SHARP_READER_SIZE,
 				      scm_sharp_reader_standard_specs,
 				      fault_handler, c_flags,
@@ -236,7 +248,8 @@ scm_load_standard_reader (void)
 	scm_c_make_reader (standard_sharp_reader_code,
 			   sizeof (standard_sharp_reader_code),
 			   scm_sharp_reader_standard_specs,
-			   scm_reader_standard_fault_handler_proc, 0,
+			   scm_variable_ref (scm_reader_standard_fault_handler_var),
+			   0,
 			   &code_size);
     }
 
@@ -253,7 +266,8 @@ scm_load_standard_reader (void)
 	scm_c_make_reader (standard_reader_code,
 			   sizeof (standard_reader_code),
 			   scm_reader_standard_specs,
-			   scm_reader_standard_fault_handler_proc, 0,
+			   scm_variable_ref (scm_reader_standard_fault_handler_var),
+			   0,
 			   &code_size);
     }
 }
