@@ -1,10 +1,10 @@
 /* A Scheme reader compiler for Guile.
 
-   Copyright (C) 2005  Ludovic Courts  <ludovic.courtes@laas.fr>
+   Copyright (C) 2005, 2009, 2012  Ludovic Courts  <ludo@gnu.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -17,8 +17,8 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA  */
 
 
-#ifndef __SCM_TOKEN_READERS_H__
-#define __SCM_TOKEN_READERS_H__
+#ifndef GUILE_TOKEN_READERS_H
+#define GUILE_TOKEN_READERS_H
 
 /* A list of specific token readers, most of which were stolen from
    Guile.  */
@@ -29,68 +29,81 @@
 
 /* Main token readers.  */
 
-extern SCM scm_read_sexp (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_sexp (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_string (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_string (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_guile_number (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_guile_number (scm_t_wchar, SCM, scm_reader_t,
+				  scm_reader_t);
 
-extern SCM scm_read_number_and_radix (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_number_and_radix (scm_t_wchar, SCM, scm_reader_t,
+				      scm_reader_t);
 
-extern SCM scm_read_quote (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_quote (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_semicolon_comment (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_semicolon_comment (scm_t_wchar, SCM, scm_reader_t,
+				       scm_reader_t);
 
-extern SCM scm_read_scsh_block_comment (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_scsh_block_comment (scm_t_wchar, SCM, scm_reader_t,
+					scm_reader_t);
 
-extern SCM scm_read_srfi30_block_comment (int, SCM, scm_reader_t,
+extern SCM scm_read_srfi30_block_comment (scm_t_wchar, SCM, scm_reader_t,
 					  scm_reader_t);
 
-extern SCM scm_read_srfi62_sexp_comment (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_srfi62_sexp_comment (scm_t_wchar, SCM, scm_reader_t,
+					 scm_reader_t);
 
-extern SCM scm_read_boolean (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_boolean (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_character (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_character (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_keyword (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_keyword (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_vector (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_vector (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_srfi4_vector (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_srfi4_vector (scm_t_wchar, SCM, scm_reader_t,
+				  scm_reader_t);
 
-extern SCM scm_read_guile_bit_vector (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_guile_bit_vector (scm_t_wchar, SCM, scm_reader_t,
+				      scm_reader_t);
 
-extern SCM scm_read_skribe_exp (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_skribe_exp (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
 
 /* Symbol token readers.  */
-extern SCM scm_read_guile_mixed_case_symbol (int, SCM, scm_reader_t,
+extern SCM scm_read_guile_mixed_case_symbol (scm_t_wchar, SCM, scm_reader_t,
 					     scm_reader_t);
 
-extern SCM scm_read_r5rs_lower_case_symbol (int, SCM, scm_reader_t,
+extern SCM scm_read_r5rs_lower_case_symbol (scm_t_wchar, SCM, scm_reader_t,
 					    scm_reader_t);
 
-extern SCM scm_read_r5rs_upper_case_symbol (int, SCM, scm_reader_t,
+extern SCM scm_read_r5rs_upper_case_symbol (scm_t_wchar, SCM, scm_reader_t,
 					    scm_reader_t);
 
-extern SCM scm_read_r5rs_lower_case_number (int, SCM, scm_reader_t,
+extern SCM scm_read_r5rs_lower_case_number (scm_t_wchar, SCM, scm_reader_t,
 					    scm_reader_t);
 
-extern SCM scm_read_r5rs_upper_case_number (int, SCM, scm_reader_t,
+extern SCM scm_read_r5rs_upper_case_number (scm_t_wchar, SCM, scm_reader_t,
 					    scm_reader_t);
 
-extern SCM scm_read_r6rs_symbol (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_r6rs_symbol (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_r6rs_number (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_r6rs_number (scm_t_wchar, SCM, scm_reader_t, scm_reader_t);
 
-extern SCM scm_read_brace_free_number (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_r6rs_syntax_quote (scm_t_wchar, SCM, scm_reader_t,
+				       scm_reader_t);
 
-extern SCM scm_read_brace_free_symbol (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_brace_free_number (scm_t_wchar, SCM, scm_reader_t,
+				       scm_reader_t);
+
+extern SCM scm_read_brace_free_symbol (scm_t_wchar, SCM, scm_reader_t,
+				       scm_reader_t);
 
 
 /* Sharp token readers, should be called after `#' has been read.  */
 
-extern SCM scm_read_extended_symbol (int, SCM, scm_reader_t, scm_reader_t);
+extern SCM scm_read_extended_symbol (scm_t_wchar, SCM, scm_reader_t,
+				     scm_reader_t);
 
 
 /* Look for a token reader named NAME in the standard token readers and
@@ -247,7 +260,7 @@ extern void scm_initialize_token_reader_library (void);
   SCM_DEFTOKEN_SINGLE ('"', "string", scm_read_string, 0,	\
 		       "Read an R5RS string.")
 #define SCM_TR_GUILE_NUMBER /* guile-number */				\
-  SCM_DEFTOKEN_RANGE ('0', '9', "guile-number",				\
+  SCM_DEFTOKEN_SET   ("-+.0123456789", "guile-number",			\
 		      scm_read_guile_number, 0,				\
 		      "Read a number following Guile's fashion, "	\
 		      "that is, as in R5RS (@inforef{Lexical "		\
@@ -271,16 +284,24 @@ extern void scm_initialize_token_reader_library (void);
 		      scm_read_guile_mixed_case_symbol, 0,		\
 		      "Read a symbol that starts with an upper-case "	\
 		      "letter in a case-sensitive fashion.")
-#define SCM_TR_GUILE_SYMBOL_MISC_CHARS /* guile-symbol-misc-chars */	 \
-  SCM_DEFTOKEN_SET ("[]{}~^:.+-/*%&@_<>!=?$",				 \
-		    "guile-symbol-misc-chars",				 \
-		    scm_read_guile_mixed_case_symbol, 0,		 \
+#define SCM_TR_GUILE_SYMBOL_MISC_CHARS /* guile-symbol-misc-chars */	\
+  SCM_DEFTOKEN_SET ("[]{}~^:/*%&@_<>!=?$",				\
+		    "guile-symbol-misc-chars",				\
+		    scm_read_guile_mixed_case_symbol, 0,		\
 		    "Read a symbol that starts with a non-alphanumeric " \
-		    "character in a case-sensitive fashion.")
+		    "character in a case-sensitive fashion.  Note that " \
+		    "this token reader does @emph{not} consider square " \
+		    "brackets as delimiters, as was the case with Guile " \
+		    "1.8 and earlier.")
 #define SCM_TR_QUOTE_QUASIQUOTE_UNQUOTE /* quote-quasiquote-unquote */	  \
   SCM_DEFTOKEN_SET ("'`,", "quote-quasiquote-unquote",			  \
 		    scm_read_quote, 0,					  \
 		    "Read a quote, quasiquote, or unquote S-expression.")
+#define SCM_TR_R6RS_SYNTAX_QUOTE_QUASIQUOTE_UNQUOTE /* r6rs-syntax-quote-quasiquote-unquote */ \
+  SCM_DEFTOKEN_SET ("'`,", "r6rs-syntax-quasiquote-unquote",		\
+		    scm_read_r6rs_syntax_quote, 0,			\
+		    "Read an R6RS-style syntax, quasisyntax, or unsyntax " \
+		    "S-expression.")
 #define SCM_TR_SEMICOLON_COMMENT /* semicolon-comment */		   \
   SCM_DEFTOKEN_SINGLE (';', "semicolon-comment",			   \
 		       scm_read_semicolon_comment, 0,			   \
@@ -330,7 +351,7 @@ extern void scm_initialize_token_reader_library (void);
 		      "letter and return a lower-case symbol, "				 \
 		      "regardless of the case of the input.")
 #define SCM_TR_R5RS_LOWER_CASE_SYMBOL_MISC_CHARS /* r5rs-lower-case-symbol-misc-chars */ \
-  SCM_DEFTOKEN_SET ("[]{}~^:.+-/*%&@_<>!=?$",						 \
+  SCM_DEFTOKEN_SET ("[]{}~^:/*%&@_<>!=?$",						 \
 		    "r5rs-lower-case-symbol-misc-chars",				 \
 		    scm_read_r5rs_lower_case_symbol, 0,					 \
 		    "Read a symbol that starts with a non-"				 \
@@ -358,7 +379,7 @@ extern void scm_initialize_token_reader_library (void);
 		      "brackets as delimiters (@pxref{Token "		\
 		      "Delimiters}).")
 #define SCM_TR_R6RS_SYMBOL_MISC_CHARS /* r6rs-symbol-misc-chars */	\
-  SCM_DEFTOKEN_SET ("{}~^:.+-/*%&@_<>!=?$",				\
+  SCM_DEFTOKEN_SET ("{}~^:/*%&@_<>!=?$",				\
 		    "r6rs-symbol-misc-chars",				\
 		    scm_read_r6rs_symbol, 0,				\
 		    "Read a symbol that starts with a non-"		\
@@ -383,7 +404,7 @@ extern void scm_initialize_token_reader_library (void);
 		      "reader recognizes braces as delimiters, "		\
 		      "unlike R5RS/R6RS.")
 #define SCM_TR_BRACE_FREE_SYMBOL_MISC_CHARS /* brace-free-symbol-misc-chars */ \
-  SCM_DEFTOKEN_SET ("[]~^:.+-/*%&@_<>!=?$",				       \
+  SCM_DEFTOKEN_SET ("[]~^:/*%&@_<>!=?$",				       \
 		    "brace-free-symbol-misc-chars",			       \
 		    scm_read_brace_free_symbol, 0,			       \
 		    "Read a symbol that starts with a non-"		       \
@@ -395,21 +416,21 @@ extern void scm_initialize_token_reader_library (void);
 /* Flavours of numbers.  */
 
 #define SCM_TR_R5RS_LOWER_CASE_NUMBER /* r5rs-lower-case-number */	\
-  SCM_DEFTOKEN_RANGE ('0', '9', "r5rs-lower-case-number",		\
+  SCM_DEFTOKEN_SET   ("-+.0123456789", "r5rs-lower-case-number",		\
 		      scm_read_r5rs_lower_case_number, 0,		\
 		      "Return a number or a lower-case symbol.")
 #define SCM_TR_R5RS_UPPER_CASE_NUMBER /* r5rs-upper-case-number */	\
-  SCM_DEFTOKEN_RANGE ('0', '9', "r5rs-upper-case-number",		\
+  SCM_DEFTOKEN_SET   ("-+.0123456789", "r5rs-upper-case-number",	\
 		      scm_read_r5rs_upper_case_number, 0,		\
 		      "Return a number or an upper-case symbol.")
 #define SCM_TR_R6RS_NUMBER /* r6rs-number */				\
-  SCM_DEFTOKEN_RANGE ('0', '9', "r6rs-number",				\
+  SCM_DEFTOKEN_SET   ("-+.0123456789", "r6rs-number",			\
 		      scm_read_r6rs_number, 0,				\
 		      "Return a number or a symbol.  This token "	\
 		      "reader conforms to R6RS, i.e. it considers "	\
 		      "square brackets as delimiters.")
 #define SCM_TR_BRACE_FREE_NUMBER /* brace-free-number */		\
-  SCM_DEFTOKEN_RANGE ('0', '9', "brace-free-number",			\
+  SCM_DEFTOKEN_SET   ("-+.0123456789", "brace-free-number",		\
 		      scm_read_brace_free_number, 0,			\
 		      "Return a number or a symbol, considering "	\
 		      "curly braces as delimiters.")			\
