@@ -1,18 +1,19 @@
 #!/bin/sh
 # aside from this initial boilerplate, this is actually -*- scheme -*- code
 LTDL_LIBRARY_PATH="`pwd`/.libs"
-export LTDL_LIBRARY_PATH
+GUILE_AUTO_COMPILE=0
+export LTDL_LIBRARY_PATH GUILE_AUTO_COMPILE
 main='(module-ref (resolve-module '\''(scheme-reader)) '\'main')'
 exec ${GUILE-"${top_builddir-..}/pre-inst-guile"} -l $0  \
         -c "(apply $main (cdr (command-line)))" "$@"
 !#
 ;;;
-;;; Copyright 2005, 2007  Ludovic Courts <ludovic.courtes@laas.fr>
+;;; Copyright 2005, 2007, 2009, 2010, 2012  Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
-;;; the Free Software Foundation; either version 2 of the License, or
+;;; the Free Software Foundation; either version 3 of the License, or
 ;;; (at your option) any later version.
 ;;;
 ;;; This program is distributed in the hope that it will be useful,
@@ -32,7 +33,7 @@ exec ${GUILE-"${top_builddir-..}/pre-inst-guile"} -l $0  \
   #:use-module (srfi srfi-11)
   #:use-module (ice-9 format))
 
-;;; Author:  Ludovic Courts
+;;; Author:  Ludovic Courtès
 ;;;
 ;;; Commentary:
 ;;;
@@ -42,6 +43,9 @@ exec ${GUILE-"${top_builddir-..}/pre-inst-guile"} -l $0  \
 ;;;
 ;;; Code:
 
+;; Have the current output port use the current locale encoding.
+(setlocale LC_ALL "")
+
 
 (define-public (read-this str reader)
   (with-input-from-string str
@@ -118,7 +122,7 @@ encountered."
 (define-public test-cases
   ;; A series of simple test cases and corner cases for the standard Scheme
   ;; reader.
-  '((string . "\"hello\"")
+  `((string . "\"hello\"")
     (string-with-backslash . "\"hello \\\"world\\\"\"")
     (symbols-with-cr . "first-symbol\rsecond-symbol")
     (number . "777")
@@ -128,6 +132,7 @@ encountered."
     (number-exponential . "1e9")
     (number-exponential-negative . "1e-09")
     (negative-number-exponential-negative . "-77e-77")
+    (rational-number . "2/3")
     (complex-number . "1+2i")
     (complex-number-floating . "1.0-3.14I")
     (complex-pure-imaginary . "+i")
@@ -142,7 +147,6 @@ encountered."
     (symbol-star . "*symbol*")
     (symbol-at . "@symbol@")
     (symbol-brace . "{symbol}")
-    (symbol-square-brackets . "[symbol]")
     (symbol-dot . ".")
     (symbol-lower-than . "<")
     (symbol-greater-than-or-equal . ">=")
@@ -152,6 +156,8 @@ encountered."
     (symbol-one-plus . "1+")
     (symbol-plus-plus-i . "1++i")
     (symbol-plus-i-plus-i . "1+i+i")
+    (symbol-like-rational . "2/")
+    (symbol-like-rational-two-slashes . "2//3")
     (symbol-like-complex . "1+e10000i")
     (symbol-like-exponential . "1e")
     (quoted-symbol . "'quoted-symbol")
@@ -162,7 +168,7 @@ encountered."
     (pair-long . "(a b . c)")
     (pair-again . "(a .b)")
     (fake-pair . "(. a)")
-    (quasiquote . "`a")
+    (quasi-quote . "`a")
     (quasiquote-unquote . "`(a ,b)")
     (quasiquote-unquote-splicing . "`(a ,@b)")
     (line-comment . ";;; line comment\n777")
@@ -189,6 +195,17 @@ encountered."
     (oct-number . "#o10")
     (dec-number . "#d10")
     (bit-vector . "#*010101010101")
+    ,@(cond-expand
+       (guile-2                         ; syntax only supported in Guile 2.0+
+        '((srfi-62-sexp-comment-within-sexp . "(+ 1 #;(+ 4 5 6) 2)")
+          (srfi-62-sexp-comment-finishing-sexp . "(+ 1 2 #;(+ 4 5 6))")
+          (syntax-quote . "#'foo")
+          (syntax-quasiquote . "#`foo")
+          (syntax-unquote . "#`(foo #,bar)")
+          (syntax-unquote-splicing . "#`(foo #,@bar)")
+          (square-bracket-sexp . "[not-a-symbol]"))) ; a list under Guile 2.0
+       (else
+        '()))
 
     ;; more complex expressions
     (pair-of-chars . "(#\\a . #\\b)")
@@ -235,8 +252,11 @@ encountered."
 
 		  ;; Try out two versions of the Guile reader: one that
 		  ;; doesn't record positions and one that does.
+                  ;; Explicitly use `%reader-standard-fault-handler' so that
+                  ;; Unicode symbols are properly read on Guile 2.
 		  (list (default-reader)
-			(make-guile-reader #f 'reader/record-positions)))
+			(make-guile-reader %reader-standard-fault-handler
+                                           'reader/record-positions)))
 	(format #t "~%~%* summary~%~%total: ~a~%failed: ~a~%"
 		total failed))
 
@@ -254,6 +274,4 @@ encountered."
 
 (define main scheme-reader)
 
-;;; arch-tag: testsuite.scm
-
 ;;; scheme-reader.scm ends here
